% dsp03_ex_transforms.m - orthogonal transformations of signals
clear all; close all

% Orthogonality concept - for vectors and matrices
v1 = [ 1 0 0 0 0 0 0 0 ];     % Vector no. 1
v2 = [ 0 1 0 0 0 0 0 0 ];     % Vector no. 2
prod1 = sum( v1 .* conj(v2) ) % If the sum of multiplication of corresponding 
prod2 = dot( v1, v2 )         % coefficients of two vectors is equal to 0,
prod3 = v1*v2'                % vectors are orthogonal in Euclidean space.
A = eye(8);                   % diagonal identity matrix as orthogonal matrix
prod4 = inv(A) * A,           % product of matrix and its inverse should give I matrix
prod5 = A' * A,               % inv(A)->A' for orthogonal matrices 

% Orthogonal matrix for DCT-IV orthogonal transform  
N = 100;                                    % transformation order (square matrix size)
k = (0:N-1); n=(0:N-1);                     % columns=functions, rows=samples
S = sqrt(2/N)*cos(pi/N*(n'+1/2)*(k+1/2));   % synthesis matrix
A = S';                                     % analysis matrix: conjugation+transposition of S
S*A, pause                                  % checking matrix orthogonality
 
x1 = 10*S(:,5);                                  % signal #1 - vertical!                
x2 = 20*S(:,10);                                 % signal #2
x3 = 30*sqrt(2/N)*cos(pi/N*(n'+1/2)*(15+1/2));   % signal #3
x4 = randn(N,1);                                 % signal #4
x = x4; % choose x1, x2, x3, x4, x1+x2, x1+x3, x1+x4 
figure; plot(x,'bo-'); title('x(n)'); grid;      % input
 
c = A*x;                 % signal analysis: finding similarity coefficients
figure; stem(c); grid;   % displaying similarity coefficients
%c(3) = 0;               % removing 3-rd signal component 
y = S*c;                 % signal synthesis: summation of weighted elementary vectors
figure; plot(y,'bo-'); title('y(n)'); grid;         % output
error = max(abs(x-y)),   % signal reconstruction error

