%dsp07_ex_analog_butter.m - analog Butterworth filters
clear all; close all;

% Design/choice of filter coefficients - low-pass Butterworth

N = 5;                                   % number of TF poles
f0 = 100;                                % cut-off frequency of low-pass filter
alpha = pi/N;                            % angle of one ``piece of cake''
beta  = pi/2 + alpha/2 + alpha*(0:N-1);  % angles of poles
R = 2*pi*f0;                             % circle radius
p = R*exp(j*beta);                       % poles placed on circle, left half-plane
%z = []; gain = prod(-p);                % LOW-PASS: TF zeros are not used, gain
z = zeros(1,N); gain = 1;                % HIGH-PASS: N TF zeros in zero, gain
b = gain*poly(z); a=poly(p);             % [z,p] --> [b,a]
b = real(b);      a=real(a);             %

% ... continuation dsp07_ex_analog_intro.m
alf=0:pi/100:2*pi; c = R*cos(alf); s = R*sin(alf);
figure; plot(real(z),imag(z),'bo', real(p),imag(p),'r*',c,s,'b-'); grid;
title('Zeros and Poles'); xlabel('Real()'); ylabel('Imag()'); pause

% Verification of filter responses: amplitude, phase, impulse, step
f = 0 : 1 : 1000;                  % frequency in hertz
w = 2*pi*f;                        % pulsation, radial frequency
s = j*w;                           % Laplace transform variable
H = polyval(b,s) ./ polyval(a,s);  % FR=TF for s=j*w: ratio of two polynomials
figure; plot(f,20*log10(abs(H))); xlabel('f [Hz]'); title('|H(f)| [dB]'); grid; pause
figure; semilogx(f,20*log10(abs(H))); xlabel('f [Hz]'); title('|H(f)| [dB]'); grid; pause
figure; plot(f,unwrap(angle(H))); xlabel('f [Hz]'); title('angle(H(f)) [rad]'); grid; pause
figure; impulse(b,a); pause        % filter response to impulse on input 
figure; step(b,a); pause           % filter responset to step change on input