% lab07_ex_hardware.m - desing of analog filter hardware (example from the book) 
clear all; clear all;

% DESIGN REQUIREMENTS
fpass =  8000;	 %  end of pass-band [Hz] corresponding to apass
fstop = 22050;	 %  begining of stop-band [Hz] corresponding to astop
apass = 2;		 %  non-linearity in the pass-band [dB]
astop = 40;		 %  attenuation in the stop-band [dB]

%  UDING MATLAB BUILT-IN FUNCTIONS 
%  [N,w0] = buttord( 2*pi*fpass, 2*pi*fstop, -apass, -astop, 's' );
%  [z,p,k] = buttap(N);
%  [b,a] = zp2tf(z,p,k);

gain_p = 10^(-apass/20)     % attenuation pass [dB] -> linear gain pass
gain_s = 10^(-astop/20)     % attenuation stop [dB] -> linear gain stop
pause
ws = fstop/fpass;           % frequency transformation: s=s'/w0, w0=2*pi*fpass
vp = 2*pi*fpass;  vs = 2*pi*fstop;
f_ps = [fpass, fstop];  gain_ps = [gain_p, gain_s];  gaindB_ps = [-apass, -astop];

disp('======= ANALOG PROTOTYPE =======')
wp = 1;
Nreal = log10( (10^(astop/10)-1) / (10^(apass/10)-1) ) / (2*log10(ws/wp))
N  = ceil( Nreal )
w0 = ws / (10^(astop/10)-1)^(1/(2*N))
pause

% Calculation of filter poles
% Poles --> coefficients of denominator polynomial A(z)
% Calculate nominator constant ensuring H(0)=B(0)/A(0)=1

dfi0 = (2*pi)/(2*N);                    %  angle of piece of cake
fi = pi/2 + dfi0/2 + (0 : N-1)*dfi0;    %  pole angles
p = w0*exp(j*fi)                        %  TF poles
z = [];                                 %  TF zeros
gain = real( prod(-p) );                %  gain/constant
pause

figure
plot( real(p), imag(p), 'xb' ); grid; title('Position of TF poles'); 
xlabel('real'); ylabel('imag'); pause

b = gain;		%  coefficients of nominator B(z) polynomial, constant 
a = poly(p);	%  poles --> coefficints of denominator A(z) polynomial

b, a, pause
printsys(b,a,'s'), pause

% Compare with Matlab functions
[NN,ww0] = buttord( vp, vs, apass, astop, 's' );
error_N = N-NN, pause

% Calculation of filter frequency response H(w)=B(w)/A(w)
w = 0 : 0.005 : 2;   % angular frequency (2*pi*f) range (cut-off=1)
H = freqs(b,a,w);    % alternatively: H = polyval( b,j*w)./polyval(a,j*w);
figure; plot(w,abs(H)); grid; title('|H(f)|'); xlabel('angular frequency [rad/s]'); pause
figure; plot(w,20*log10(abs(H))); grid; title('|H(w)| [dB]');
xlabel('angular frequency [rad/s]'); ylabel('dB'); pause

% Frequency transformation of analog normalized prototype to target analog filter
[z,p,gain] = lp2lpTZ(z,p,gain,vp);   %  LowPass to LowPass: s=s/w0

b = gain*poly(z); a = poly(p); 

% Show TF H(z) zeros and poles after frequency transformation
figure; plot( real(z), imag(z), 'or',real(p),imag(p),'xb' ); grid;
title('Position of TF poles'); xlabel('real'); ylabel('imag');
pause

disp('======= TARGET ANALOG FILTER AFTER FREQ TRANSFORMATION =======')

z, p, pause
b, a, pause
printsys(b,a,'s'), pause

% Final, obtained frequency response
NF = 1000;                              %  number of frequency points
fmin = 0;                               %  minimum frequency
fmax = 50000;                           %  maximum frequency
f = fmin : (fmax-fmin)/(NF-1) : fmax;   %  all frequency values
w = 2*pi*f;                             %  all radial frequency values
H = freqs(b,a,w);    %  alternatively: H = polyval( b,j*w)./polyval(a,j*w);
figure; plot( f,abs(H), f_ps, gain_ps,'ro');
grid; title('|H(f)|'); xlabel('freq [Hz]'); pause
figure; plot(f, 20*log10(abs(H)), f_ps, gaindB_ps,'ro');
axis([fmin,fmax,-100,20]); grid; title('|H(f)| [dB]'); 
xlabel('freq [Hz]'); ylabel('dB'); pause
figure; plot(f,unwrap(angle(H))); grid; title('angle( H(f) )');
xlabel('freq [Hz]'); ylabel('[rd]'); pause

disp('======= ACTIVE ANALOG RC FILTER =======')

% Calculation of circuit elements, 

p(1), p(2), p(5), pause

% Find conjugated pairs
p1 = [ p(1) conj(p(1)) ];
p2 = [ p(2) conj(p(2)) ];
p3 = p(3);

aw1 = poly(p1), aw2 = poly(p2), aw3 = poly(p3), pause

C = 10^(-9); RA=10^4;  Rout = 10^4;

disp('=== Section 1===')
a = aw1;
a2=a(1); a1=a(2); a0=a(3);
R = 1/(C*sqrt(a0))
RB = (2-a1/sqrt(a0)) * RA
K1 = 1+RB/RA
pause

disp('=== Section 2 ===')
a = aw2;
a2=a(1); a1=a(2); a0=a(3);
R = 1/(C*sqrt(a0))
RB = (2-a1/sqrt(a0)) * RA
K2 = 1+RB/RA
pause

disp('=== Section 3 ===')
a = aw3;
a1=a(1); a0=a(2);
R=1/(C*a0)
K3=1
pause

disp('=== Load ===')
K=K1*K2*K3         % actual gain
G=1                % required gain
Rx = (K/G)*Rout
Ry = K/(K-G)*Rout
pause
