%dsp8_ex_iir_intro.m
clear all; close all;

% Design/choice of filter coefficients
fs = 2000;                               % sampling frequency
if(0)                                    % choosing polynomial coefficients
   b = [2,3];                            % [ b0, b1 ]
   a = [1, 0.2, 0.3, 0.4];               % [ a0=1, a2, a3, a4]
   z = roots(b); p = roots(a);           % [b,a] --> [z,p]
else                                     % choosing polynomial roots
   gain = 0.001;
   z = [ 1,   1    ] .* exp(j*2*pi*[ 600, 800 ]/fs);   z = [z conj(z)];  
   p = [ 0.99,0.99 ] .* exp(j*2*pi*[ 100, 200 ]/fs);   p = [p conj(p)];
   b = gain*poly(z), a = poly(p), pause       % [z,p] --> [b,a]
end
figure;
v=0:pi/1000:2*pi; c=cos(v); s=sin(v);
plot(real(z),imag(z),'bo', real(p),imag(p),'r*',c,s,'k-'); grid;
title('Zeros and Poles'); xlabel('Real()'); ylabel('Imag()'); pause

% Verification of filter responses: amplitude, phase, impulse, step
f = 0 : 0.1 : 1000;                 % frequency in hertz
om = 2*pi*f/fs;                     % normalized pulsation, radial frequency
zz = exp(-j*om);                    % Z transform variable z^(-1)
H = polyval(b(end:-1:1),zz) ./ polyval( a(end:-1:1),zz); % ratio of two polynomials
% H = freqz(b,a,f,fs)                                    % all-in-one Matlab function
figure; plot(f,20*log10(abs(H))); xlabel('f [Hz]'); title('|H(f)| [dB]'); grid; pause
figure; plot(f,unwrap(angle(H))); xlabel('f [Hz]'); title('angle(H(f)) [rad]'); grid; pause

% ... continuation of dsp8_ex_iir_intro.m

% Input signal x(n) - two sinusoids: 20 and 500 Hz
Nx = 1000;                                       % number of samples
dt = 1/fs; t = dt*(0:Nx-1);                      % sampling times
%x = zeros(1,Nx); x(1) = 1;                      % Kronecker delta impulse
x = sin(2*pi*20*t+pi/3) + sin(2*pi*500*t+pi/7);  % sum of 2 sines

% Digital IIR filtration: x(n) --> [ b, a ] --> y(n)
% y=filter(b,a,x);                           % all-in-one Matlab function
M = length(b);                               % number of {b} coefficients
N = length(a); a = a(2:N); N=N-1;            % number of {a} coeffs, remove a0=1
bx = zeros(1,M);                             % buffer for input samples x(n)
by = zeros(1,N);                             % buffer for output samples y(n)
for n = 1 : Nx                               % MAIN LOOP
    bx = [ x(n) bx(1:M-1) ];                 % put new x(n) into bx buffer
    y(n) = sum( bx .* b ) - sum( by .* a );  % do filtration, find y(n)
    by = [ y(n) by(1:N-1) ];                 % put y(n) into by buffer
end
  
% FIGURES: comparison of input and output
figure;
subplot(211); plot(t,x); title('x(t)'); grid;  % input  signal x(t)
subplot(212); plot(t,y); title('y(t)'); grid;  % output signal y(t)
xlabel('t [s]'); pause
figure; % signal spectra of the second halves of samples (transients are removed!)
k=Nx/2+1:Nx;  f0 = fs/(Nx/2);  f=f0*(0:Nx/2-1); 
subplot(211); plot(f,20*log10(abs(2*fft(x(k)))/(Nx/2))); title('|X(f)|'); grid; 
subplot(212); plot(f,20*log10(abs(2*fft(y(k)))/(Nx/2))); title('|Y(f)|'); grid;
xlabel('f [Hz]'); pause