% dsp09_ex_invdft.m - FIR filter design using inverse DFT method
clear all; close all;

% Parameters
fs = 2000;               % sampling frequency (Hz)
f0=100; f1=400; f2=600;  % cut-off frequencies
M = 100;                 % half of the filter weights
K = 4;                   % oversampling order in frequency domain
N = 2*M + 1;             % number of weights {b(n)} - always odd
n = -M :1: M;            % non-causal weights indexes
NK = N*K; if(rem(NK,2)==1) NK=NK+1; end   % after oversampling in freq
df = fs/NK; f = df*(0 : NK/2);            % frequencies
H0 = zeros(1,NK/2+1);                     % initialization
H1 = ones(1,NK/2+1);                      % initialization
% Low-Pass
ind = find( f<=f0 );     
HLP = H0; HLP(ind) = ones(1,length(ind)); HLP(ind(end))=0.5;
% High-Pass
ind = find( f>=f0 );
HHP = H0; HHP(ind) = ones(1,length(ind)); HLP(ind(1))=0.5;
% Band-Pass and Band-Stop
ind1 = find( f< f1 );    
ind2 = find( f<=f2 );
ind  = find( ind2 > ind1(end) );
HBP = H0; HBP(ind) = ones(1,length(ind));  HBP(ind(1))=0.5; HBP(ind(end))=0.5; 
% HBS = ? 

% Our choice: LP, HP, BP, BS
H = HBP;                                        % choice: LP, HP, BP, BS
H(NK:-1:NK/2+2) = H(2:NK/2);                    % second half by symmetry

h = ifft( H );                                  % inverse DFT
b = [ h(M+1:-1:2) h(1:M+1) ];                   % selection of 2M+1 weights
%b = b .* chebwin(N,100)';                      % optional windowing
figure; stem(n,b); title('b(n)'); xlabel('n'); grid; pause   % figure
