% dsp09_ex_ls.m - FIR filter design using weighted Least Squares (LS) method
clear all; close all;

% Parameters  
fs = 2000;               % sampling frequency (Hz)
f0 = 100;                % cut-off frequency
M = 100;                 % half of the filter weights, N=2M+1
K = 4;                   % oversampling order in frequency domain
% P points of frequency response Ad() for pulsations 2*pi*p/P, p=0,1,...,P-1
P = K*2*M;               % number of points (even; P >= N=2M+1)
L1 = floor(f0/fs*P),     % number of first FR samples equal gain=1
Ad = [ ones(1,L1) 0.5 zeros(1,P-(2*L1-1)-2) 0.5 ones(1,L1-1)]; 
Ad = Ad'; 
% Chose weighting coeffs w(p), p=0,1,2,...,P-1, for Pass/Trans/Stop regions
wp = 1;                  %  weight for PassBand
wt = 1;                  %  weight for TransientBand
ws = 10000;              %  weight for StopBand
w = [ wp*ones(1,L1) wt ws*ones(1,P-(2*L1-1)-2) wt wp*ones(1,L1-1) ]; 
W = zeros(P,P);          % 
for p=1:P                %  matrix with weights on the main diagonal
    W(p,p)=w(p);         %
end                      %
% Find matrix F being solution of the matrix equation  W*F*h = W*(Ad + err) 
F = [];
n = 0 : M-1; 
for p = 0 : P-1
    F = [ F; 2*cos(2*pi*(M-n)*p/P) 1 ]; 
end
% Find h(n) minimizing error of W*F*h = W*Ad
% h = pinv(W*F)*(W*Ad);      % method #1
h = (W*F)\(W*Ad);            % method #2
b = [ h; h(M:-1:1) ]';       % symmetrical extension
%b = b .* chebwin(N,100)';   % optional addition of arbitrary window 
figure; stem(-M:M,b); title('b(n)'); xlabel('n'); grid; pause   % figure
