
% dsp11_ex_diff.m - FIR filter-based signal differentiation
clear all; close all;

% Differentiation filter weights
M=50; N=2*M+1; n=-M:M; h=cos(pi*n)./n; h(M+1)=0;  % wights derived mathematically 
h = h .* kaiser(N,10)';                           % windowing
%h = 1/12 * [-1, 8, 0, -8, 1]; M=2; N=2*M+1; n = -M:M; % 1/2*[-1 0 1]
stem(n,h); title('h(n)'); grid; pause

% Verification of filter responses: amplitude, phase, impulse, step
fs = 2000; f = 0:1:fs/2;        % sampling frequency [Hz], frequencies of interest 
z = exp(-j*2*pi*f/fs);          % Z transform variable z^(-1)
H = polyval(h(end:-1:1),z);     % FR=TF for z: polynomial value
% H = freqz(h,1,f,fs);          % all-in-one Matlab function
figure; plot(f,abs(H)); xlabel('f [Hz]'); title('|H(f)|'); grid; pause
figure; plot(f,unwrap(angle(H))); xlabel('f [Hz]'); title('angle(H(f)) [rad]'); grid; pause
figure; plot(f,angle(exp(j*2*pi*f/fs*M).*H)); xlabel('f [Hz]'); title('angle(H(f)) [rad]'); grid; pause

% Filtering - signal differentiation
Nx=400; n=0:Nx-1; dt=1/fs; t=dt*n;
fx=100; x=sin(2*pi*fx*t);              % signal
y=filter(h,1,x);                       % application of differentiation filter 
nx=M+1:Nx-M; ny=2*M+1:Nx;
figure; plot(nx,x(nx),'ro-',nx,y(ny),'bo-'); title('x(n), y(n)'); grid; pause
