% dsp11_ex_hilb.m
clear all; close all;

% Hilbert filter and analytic signal calculation in frequency domain

Nx=1000; fs=2000; f0=fs/40; x=cos(2*pi*(f0/fs)*(0:Nx-1));
xa1=hilbert(x);                  % Matlab function
X=fft(x);                        % signal FFT, next its modification
n=1:Nx/2;  X(n)=-j*X(n);         % modification for positive frequencies
X(1)=0; X(Nx/2+1)=0;             %
n=Nx/2+2:Nx; X(n)= j*X(n);       % modification for negative frequencies
xH=real(ifft(X));                % inverse FFT 
xa2=x+j*xH;                      % analytic signal
err1=max(abs(xa1-xa2)),          % error vs Matlab
figure; plot(1:Nx,x,'ro-',1:Nx,xH,'bo-'); grid; pause
figure; plot(x,xH,'bo-'); grid; pause

% Design of Hilbert filter weights

% Weights h are derived mathematically
M=50; N=2*M+1; n=-M:M; h=(1-cos(pi*n))./(pi*n); h(M+1)=0;
figure; stem(n,h); title('h(n)'); xlabel('n'); grid; pause
% Calculation of Kaiser window and filter weigths windowing
w=kaiser(N,10)'; h = h.*w;
figure; stem(n,h); title('h(n)'); xlabel('n'); grid; pause
% Filter frequency response
fs=2000; f=-fs/2:fs/2000:fs/2;
H1 = polyval( h(end:-1:1), exp(-j*2*pi*f/fs) );
H2 = freqz(h,1,f,fs);
figure; plot( f, 20*log10(abs(H2)) ); grid; pause
figure; plot( f, unwrap(angle(H2)) ); grid; pause
figure; plot( f, angle(exp(j*2*pi*f/fs*M).*H2) ); grid; pause

% Signal filtering                   

xH = filter(h,1,x);
x = x(M+1 : Nx-M);
xH = xH(2*M+1 : Nx);
xa = x + j*xH; Nx = length(xa);
figure; plot(1:Nx,x,'ro-',1:Nx,xH,'bo-'); grid; pause
figure; plot(x,xH,'bo-'); grid; pause

% Example of AM and FM signal demodulation

Nx=2000; n=0:Nx-1; dt=1/fs; t=dt*n;

A=10; kA=0.5; fA=2; xA = A*(1 + kA*sin(2*pi*fA*t));             % AM
kF=250; fF=1;       xF = kF*sin(2*pi*fF*t);                     % FM
fc=500;             x = xA .* sin(2*pi*(fc*t + cumsum(xF)*dt)); % modulated carrier
xa = hilbert( x );                                              % analytic signal
xAest = abs( xa );                                              % AM demodulation
ang = unwrap(angle( xa ));                                      % below FM demod
xFest = (1/(2*pi)) * (ang(3:end)-ang(1:end-2)) / (2*dt);
figure; plot(t,xA,'r-',t,xAest,'b-'); title('AM'); grid;
figure; plot(t,xF,'r-',t(2:Nx-1),xFest-fc,'b-'); title('FM'); grid;
